import hudson.model.Build
import hudson.model.Result
import hudson.scm.ChangeLogSet
import hudson.scm.SubversionChangeLogSet.LogEntry

logLevelDebug = false // log debug
encoding = "ISO-8859-1" // Latin-1

// see https://gist.github.com/lyuboraykov/3a2af9bb6855db86eff0
out = getBinding().getVariables()['out']

def log(msg) {
    out.println("[jenkins] ${msg}")
}

def debugLog(msg) {
    if (logLevelDebug) {
        log("DEBUG: ${msg}")
    }
}

def changedFiles() {
    def Build build = Thread.currentThread()?.executable
    def List<String> changedFiles = changedFileSinceLastSuccessfull(build)
    changedFiles.unique().sort()
}

def changedFileSinceLastSuccessfull(Build build) {
    // see http://stackoverflow.com/a/38206961/104143
    if (build == null || build.result == Result.SUCCESS) {
        debugLog("Stopping at build '${build}' which was successful (or empty)")
        []
    } else {
        log("Collecting changes of build '${build}'")
        changedFilesIn(build) + changedFileSinceLastSuccessfull(build.getPreviousBuild())
    }
}

def changedFilesIn(Build build) {
    // see http://stackoverflow.com/a/12939663/104143
    def ChangeLogSet<LogEntry> changeSet = build.getChangeSet()
    def List<LogEntry> changedItems = changeSet.getItems()
    debugLog("${changedItems.size()} changed items")
    def changedPaths = changedItems.collect { logEntry -> logEntry.paths }.flatten()
    debugLog("${changedPaths.size()} changed paths")
    def changedFiles = changedPaths.collect { path -> path.path }
    debugLog("${changedFiles.size()} changed files")
    changedFiles
}

def save(List<String> fileNames) {
    def File lstFile = new File("${workspace()}/changed_files.log")
    log("Saving list of files names to ${lstFile}")
    lstFile.withWriter(encoding){ file ->
        fileNames.each { name -> file.println name }
    }
}

def workspace() {
    def Build build = Thread.currentThread()?.executable
    build.getWorkspace().toString()
}

log('Collecting changes files')
save(changedFiles())
